
	ifd MDL_CLONING

;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_CLONE_OBJECT
; Description : Clone un objet et sa descendance
;
; Rq  : Les pointeurs dsignant l'arborescence sont dfinis en absolu dans l'objet
; Rq  :  cloner. Ils seront relogs automatiquement pour le clone.
; Rq  : Le pointeur sur le pre est positionn automatiquement s'il ne l'est
; Rq  : pas dj. Autrement il est relog.
; Rq  : Aprs le delete de l'objet, on doit faire un MFREE de cet objet.
;
; in  : a0.l = pointeur sur l'objet  cloner
; in  : a1.l = pointeur sur une table d'offset dsignant les champs  reloger termine par GWVA_ROOT_CTE_END_OFFSET_LIST, 0.l si pas
; in  : d0.l = taille de la zone  cloner
; in  : d1.w = GWVA_ROOT_CTE_CLONE_RELOC_ONE_OBJECT reloge seulement l'objet pre
;	   GWVA_ROOT_CTE_CLONE_RELOC_FAMILY_TREE reloge en plus toute la descendance
; out : a0.l = pointeur sur le clone de l'objet
; out : d7.w = GWVA_NO_ERROR_GENERIC, ou erreur
; 	   GWVA_ERROR_CLONING_RELOCATION_TREE
; 	   GWVA_ERROR_CLONING_OBJECT_ALREADY_CREATED
;	   GWVA_ERROR_CLONING_SIZE
;	   GWVA_ERROR_MALLOC
;
; 22/08/98 : Cration
;------------------------------------------------------------------------------
GWVA_ROOT_CLONE_OBJECT:

; Ne pas cloner un objet dj cr !

	save.l d0-d1/a1
	bsr GWVA_ROOT_TEST_IF_OBJECT_CREATED
	load.l d0-d1/a1
	cmp.w #GWVA_NO_ERROR_GENERIC,d7
	beq .erreur_objet_deja_cree

	cmp.l #GWVA_OBJ_STRUCT_LENGTH,d0
	blt .pas_clonnage

	move.l d0,d2
	btst.l #0,d2
	beq.s .taille_paire

	add.l #1,d2
.taille_paire:

	save.l a0-a1/d0-d1
	MXALLOC #MX_PREFTTRAM,d2
	move.l d0,d2
	load.l a0-a1/d0-d1
	tst.l d2
	beq .erreur_malloc	; plus de mmoire

	move.l d2,a2	; a2.l : adresse de recopie du clone

	save.l d0/a0-a2

.recopie_zone:
	move.b (a0)+,(a2)+
	sub.l #1,d0
	bne.s .recopie_zone

	load.l d0/a0-a2

	cmp.l #0,a1
	beq.s .pas_relocation

	move.l a2,d0
.reloge_structures:
	move.l a2,a3
	add.l (a1)+,a3
	add.l d0,(a3)
	cmp.l #GWVA_ROOT_CTE_END_OFFSET_LIST,(a1)
	bne.s .reloge_structures

.pas_relocation:

	cmp.w #GWVA_ROOT_CTE_CLONE_RELOC_ONE_OBJECT,d1
	beq.s .pas_relocation_arborescence

	save.l a0/a2

	move.l a2,d0
	move.l a2,d2
	sub.l a0,d2		; valeur  ajouter pour reloger les champs relatifs  l'arborescence

	move.l a2,a0

	tst.l GWVA_OBJ_FIRST_SON_PTR(a0)
	beq.s .pas_de_fils

.objet_a_un_fils:
	move.l a0,a3	; a3 = pre

	add.l d2,GWVA_OBJ_FIRST_SON_PTR(a0)
	move.l GWVA_OBJ_FIRST_SON_PTR(a0),a0

.cherche_fils:
	tst.l GWVA_OBJ_FATHER_PTR(a0)
	bne.s .pointeur_sur_pere

	; Si il n'y a pas de pointeur sur le pre, on le met ...

	move.l a3,GWVA_OBJ_FATHER_PTR(a0)
	sub.l d2,GWVA_OBJ_FATHER_PTR(a0)	; pour viter de faire un bra.s ...

.pointeur_sur_pere:
	add.l d2,GWVA_OBJ_FATHER_PTR(a0)

	tst.l GWVA_OBJ_FIRST_SON_PTR(a0)
	bne.s .objet_a_un_fils

.cherche_frere:
	tst.l GWVA_OBJ_NEXT_BROTHER_PTR(a0)
	beq.s .pas_de_frere

	add.l d2,GWVA_OBJ_NEXT_BROTHER_PTR(a0)
	move.l GWVA_OBJ_NEXT_BROTHER_PTR(a0),a0

	bra.s .cherche_fils

.pas_de_frere:
	move.l GWVA_OBJ_FATHER_PTR(a0),a0
	move.l GWVA_OBJ_FATHER_PTR(a0),a3	; a3 = pere de ce niveau
	cmp.l a2,a0
	bne.s .cherche_frere
	
.pas_de_fils:
	load.l a0/a2
.pas_relocation_arborescence:
	move.l a2,a0
	moveq #GWVA_NO_ERROR_GENERIC,d7
	rts
.erreur_pas_de_pere:
	movem.l (sp),a0/a2
	MFREE a2
	load.l a0/a2
	move.w #GWVA_ERROR_CLONING_RELOCATION_TREE,d7
	rts

.erreur_objet_deja_cree:
	move #GWVA_ERROR_CLONING_OBJECT_ALREADY_CREATED,d7
	rts
.erreur_malloc:
	moveq #GWVA_ERROR_MALLOC,d7
	rts
.pas_clonnage:
	move #GWVA_ERROR_CLONING_SIZE,d7
	rts

	endc	; ifd MDL_CLONING

	ifnd MDL_TOS

;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_RETURN_NEXT_FAMILY_MEMBER
; Description : Renvoie l'objet suivant de l'arborescence
;
; Rq  : On parcourt d'abord tous les objets fils.
; Rq  : d0.w est in/d-crmenter si on descend/monte dans l'arborescence
;
; in  : a0.l = pointeur sur l'objet
; in  : d0.w = niveau de hirarchie (0 pour l'objet initial) [optionnel]
; out : a0.l = pointeur sur l'objet suivant
; out : d0.w = niveau courant mis  jour
; out : d7.w = GWVA_NO_ERROR_GENERIC, ou erreur
; 	   GWVA_ERROR_GENERIC
;
; 01/09/98 : Cration
;------------------------------------------------------------------------------
GWVA_ROOT_RETURN_NEXT_FAMILY_MEMBER:

	move.l GWVA_OBJ_FIRST_SON_PTR(a0),d1
	bne.s .objet_fils_existe

.test_objet_frere:
	move.l GWVA_OBJ_NEXT_BROTHER_PTR(a0),d1
	bne.s .objet_frere_existe

	move.l GWVA_OBJ_FATHER_PTR(a0),d1
	bne.s .objet_pere_existe

	move.w #GWVA_ERROR_GENERIC,d7
	rts

.objet_pere_existe:
	subq #1,d0			; remonte 1 niveau
	move.l d1,a0
	bra.s .test_objet_frere

.objet_fils_existe:
	addq #1,d0			; descend 1 niveau en profondeur
.objet_frere_existe:
	move.l d1,a0
	move.w #GWVA_NO_ERROR_GENERIC,d7
	rts

;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_RETURN_OBJ_PTR_FROM_HANDLE
; Description : Retourne un pointeur sur un objet de l'arbre  partir de son handle
;
; Rq  : On parcourt d'abord tous les objets fils.
; Rq  : d0.w est in/d-crmenter si on descend/monte dans l'arborescence
;
; in  : d0.w = handle de l'objet  rechercher dans l'arbre VA
; out : d0.w = level ou se trouve l'objet (0 pour WAPP, 1 pour win GEM ...)
; out : a0.l = pointeur sur l'objet si D7=GWVA_NO_ERROR_GENERIC
; out : d7.w = GWVA_NO_ERROR_GENERIC ou GWVA_ERROR_GENERIC
;
; 07/11/98 : Cration
;------------------------------------------------------------------------------
GWVA_ROOT_RETURN_OBJ_PTR_FROM_HANDLE:

	move d0,d1

	clr.w d0			; level
	move.l GWVA_ROOT_PTR_WAPP,a0	; 1er objet

.search_next_for_handle:
	cmp.w GWVA_OBJ_HANDLE(a0),d1
	beq.s .found

	save.w d1
	bsr GWVA_ROOT_RETURN_NEXT_FAMILY_MEMBER
	load.w d1

	cmp.w #GWVA_ERROR_GENERIC,d7
	bne.s .search_next_for_handle
	rts

.found:	moveq #GWVA_NO_ERROR_GENERIC,d7
	rts

;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_REMOVE_OBJECT_FROM_TREE
; Description : Enlve un objet de l'arborescence
;
; in  : a0.l = pointeur sur l'objet
; out : d7.w = GWVA_NO_ERROR_GENERIC ou 
;	   GWVA_ERROR_GENERIC
;
; 22/08/98 : Cration
;------------------------------------------------------------------------------
GWVA_ROOT_REMOVE_OBJECT_FROM_TREE:

	move.l GWVA_OBJ_FATHER_PTR(a0),a1
	move.l GWVA_OBJ_FIRST_SON_PTR(a1),a2

.parse_all_brothers:
	cmp.l #0,a2
	beq.s .pas_trouve
	cmp.l a2,a0
	beq.s .trouve_on_l_enleve
	
	move.l a2,a1
	move.l GWVA_OBJ_NEXT_BROTHER_PTR(a1),a2
	bra.s .parse_all_brothers

.trouve_on_l_enleve:
	cmp.l GWVA_OBJ_FATHER_PTR(a0),a1
	bne.s .pas_premier_fils

	move.l GWVA_OBJ_NEXT_BROTHER_PTR(a2),GWVA_OBJ_FIRST_SON_PTR(a1)
	bra.s .parentee_effacee

.pas_premier_fils:
	move.l GWVA_OBJ_NEXT_BROTHER_PTR(a2),GWVA_OBJ_NEXT_BROTHER_PTR(a1)

.parentee_effacee:
	clr.l GWVA_OBJ_FATHER_PTR(a2)
	clr.l GWVA_OBJ_NEXT_BROTHER_PTR(a2)
	move #GWVA_NO_ERROR_GENERIC,d7
	rts

.pas_trouve:
	move #GWVA_ERROR_GENERIC,d7
	rts

;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_CHANGE_OBJECT_IN_TREE
; Description : Met un objet  la place d'un autre dans l'arborescence objet
;
; Rq  : -
;
; in  : a0.l = pointeur sur l'objet  remplacer
; in  : a1.l = pointeur sur l'objet  mettre dans l'arborescence
; out : d7.w = GWVA_NO_ERROR_GENERIC si pas d'erreur
; 	   GWVA_ERROR_GENERIC sinon
;
; 26/07/98 : Cration
;------------------------------------------------------------------------------
GWVA_ROOT_CHANGE_OBJECT_IN_TREE:
	save.l a1
	bsr GWVA_ROOT_SEARCH_CLOSEST_PARENT_OBJECT	; -> a1.l
	load.l a2
	cmp.w #GWVA_ERROR_ROOT_BAD_CONSTRUCTION,d7
	beq.s .erreur_recherche_objet

	cmp.w #GWVA_ROOT_CTE_CLOSEST_PARENT_IS_FATHER,d7
	beq.s .le_pere

*	cmp.w #GWVA_ROOT_CTE_CLOSEST_PARENT_IS_BROTHER,d7
*	beq.s .le_frere

	; ya pas d'autre choix : c'est forcment le frre
	move.l a2,GWVA_OBJ_NEXT_BROTHER_PTR(a1)
	bra.s .le_frere

.le_pere:	move.l a2,GWVA_OBJ_FIRST_SON_PTR(a1)	; je deviens son 1er fils
.le_frere:
	move.l GWVA_OBJ_NEXT_BROTHER_PTR(a0),GWVA_OBJ_NEXT_BROTHER_PTR(a2)
	move.l GWVA_OBJ_FATHER_PTR(a0),GWVA_OBJ_FATHER_PTR(a2)
	move.w #GWVA_NO_ERROR_GENERIC,d7
	rts

.erreur_recherche_objet:
	move.w #GWVA_ERROR_GENERIC,d7
	rts
	
;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_SEARCH_CLOSEST_PARENT_OBJECT
; Description : Recherche mon objet parent le plus direct, c'est a dire mon
; Description : pre si je suis le premier fils, mon grand frre sinon
;
; Rq  : -
;
; in  : a0.l = pointeur sur l'objet
; out : d7.w = GWVA_ROOT_CTE_CLOSEST_PARENT_IS_FATHER si c'est le pre
;	   GWVA_ROOT_CTE_CLOSEST_PARENT_IS_BROTHER si c'est un frre
; 	   GWVA_ERROR_ROOT_BAD_CONSTRUCTION si erreur
;
; 26/07/98 : Cration
;------------------------------------------------------------------------------
GWVA_ROOT_SEARCH_CLOSEST_PARENT_OBJECT:
	move.l a0,a1
	move.l GWVA_OBJ_FATHER_PTR(a1),a1	; le pre de l'objet
	cmp.l #0,a1
	beq.s .erreur_pas_de_pere
	move.l GWVA_OBJ_FIRST_SON_PTR(a1),a2	; le premier fils de l'objet pre
	move.w #GWVA_ROOT_CTE_CLOSEST_PARENT_IS_FATHER,d7
.recherche_objet:
	cmp.l #0,a2
	beq.s .erreur_de_construction_arborescence	; 
	cmp.l a0,a2
	beq.s .objet_trouve
	move.l a2,a1
	move.l GWVA_OBJ_NEXT_BROTHER_PTR(a1),a2
	move.w #GWVA_ROOT_CTE_CLOSEST_PARENT_IS_BROTHER,d7
	bra.s .recherche_objet
.objet_trouve:
; En sortie, a1.l pointe sur l'objet qui pointe directement sur l'objet a0.l
	rts
.erreur_de_construction_arborescence:
.erreur_pas_de_pere:
	move.w #GWVA_ERROR_ROOT_BAD_CONSTRUCTION,d7
	rts

;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_DELETE_ALL_FIRST_SONS
; Description : Dtruit tous les fils d'un objet
;
; Rq  : a0 est sauvegard
;
; in  : a0.l = pointeur sur l'objet pre des fils  deleter
;
; 27/02/98 : Cration
;------------------------------------------------------------------------------
GWVA_ROOT_DELETE_ALL_FIRST_SONS:
	save.l a0

	move.l GWVA_OBJ_FIRST_SON_PTR(a0),a0
	cmp.l #0,a0
	beq.s .dernier_delete

.delete_tous:
	GWVA_CALL_METHOD_OTHER_OBJ	#GWVA_METHOD_DELETE

	move.l GWVA_OBJ_NEXT_BROTHER_PTR(a0),a0
	cmp.l #0,a0
	bne.s .delete_tous

.dernier_delete:
	load.l a0
	rts

;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_CREATE_ALL_FIRST_SONS
; Description : Cre tous les fils d'un objet
;
; Rq  : a0 est sauvegard
;
; in  : a0.l = pointeur sur l'objet pre des fils  crer
; out : d7.w = GWVA_NO_ERROR_GENERIC ou erreur du create du fils qui a caus l'erreur
;
; 28/03/98 : Cration
;------------------------------------------------------------------------------
GWVA_ROOT_CREATE_ALL_FIRST_SONS:
	save.l a0

	move.l GWVA_OBJ_FIRST_SON_PTR(a0),a0
	cmp.l #0,a0
	beq.s .dernier_cree

.creee_tous:
	; initialiser son pre mme si dj fait
	move.l (sp),GWVA_OBJ_FATHER_PTR(a0)

	bsr GWVA_ROOT_CREATE_OBJECT
	cmp.w #GWVA_NO_ERROR_GENERIC,d7
	bne.s .erreur_creation_fils

	move.l GWVA_OBJ_NEXT_BROTHER_PTR(a0),a0
	cmp.l #0,a0
	bne.s .creee_tous
.dernier_cree:
	load.l a0
	moveq #GWVA_NO_ERROR_GENERIC,d7
	rts

.erreur_creation_fils:
	load.l a0
	rts

	endc	; ifnd MDL_TOS

 ; car on peut envoyer des messages sans pour autant avoir besoin de MDL_EVENTS (->logger)
	ifd MDL_GEM

	ifd OPT_GWVA_WIND_REDRAW_HIDE_MOUSE

;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_HIDE_MOUSE_XYWH
; Description : Mthode pour effacer la souris si elle est dans  la
; Description : zone passe en entre.
;
; in  : a0.l = pointeur sur le buffer contenant la zone dfinie en X,Y,W,H
; out : d6.w = GWVA_PRGRET_CONSUMED
; 
; 01/11/98 : Cration
;------------------------------------------------------------------------------
GWVA_ROOT_HIDE_MOUSE_XYWH:
	save.l a0
	graf_mkstate
	load.l a0
	movem.w int_out+1*2,d4-d5	; x,y de la souris
	movem.w (a0),d0-d3
	add.w d0,d2
	add.w d1,d3
	sub.w #1,d2
	sub.w #1,d3
	sub.w #16,d0
	sub.w #16,d1
	add.w #16,d2
	add.w #16,d3
	cmp.w d4,d0
	bgt.s .no_hide_mouse
	cmp.w d4,d2
	blt.s .no_hide_mouse
	cmp.w d5,d1
	bgt.s .no_hide_mouse
	cmp.w d5,d3
	blt.s .no_hide_mouse
	bsr GWVA_ROOT_HIDE_MOUSE
.no_hide_mouse:
	rts

	endc	; ifd OPT_GWVA_WIND_REDRAW_HIDE_MOUSE

;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_HIDE_MOUSE
; Description : Efface la souris
;
; Rq  : Tiens compte d'un flag interne pour savoir si la souris est dj efface.
;
; in  : -
; out : -
;
; 01/11/98 : Cration
;------------------------------------------------------------------------------
GWVA_ROOT_HIDE_MOUSE:
	tst.w GWVA_ROOT_MOUSE_STATE
	beq.s .deja_efface
	graf_mouse #M_OFF
	clr.w GWVA_ROOT_MOUSE_STATE
.deja_efface:
	rts

;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_SHOW_MOUSE
; Description : Affiche la souris
;
; Rq  : Tiens compte d'un flag interne pour savoir si la souris est dj affiche
;
; in  : -
; out : -
;
; 01/11/98 : Cration
;------------------------------------------------------------------------------
GWVA_ROOT_SHOW_MOUSE:
	tst.w GWVA_ROOT_MOUSE_STATE
	bne.s .deja_affiche
	graf_mouse #M_ON
	move.w #1,GWVA_ROOT_MOUSE_STATE
.deja_affiche:
	rts

;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_SEND_GEM_MESSAGE
; Description : Envoie un message  une application via le pipe GEM
;
; Rq  : Des buffers temporaires sont  votre disposition :
; Rq  : ils se nomment GWVA_ROOT_TMP_MSG_OUT_BUF et GWVA_ROOT_TMP_MSG_OUT_EXT_BUF.
; Rq  : l'appelant ne doit remplir que le champs bmsg_msgid.
; Rq  : La fonction se charge de remplir les champs 'bmsg_apid' et 'bmsg_extd'
; Rq  : du message.
; Rq  : PATCH_GWVA_ROOT_GEM_MESSAGE_PADDED16 : si la longueur passe en entre 
; Rq  : n'est pas un multiple de 16 octets, on arrondit au multiple de 16 
; Rq  : suprieur : on envoie plus que ce qui est demand.
;
; in  : a0.l = pointeur sur un buffer contenant le message GEM
; in  : a1.l = pointeur sur un chaine contenant le nom de l'application 
;	   destinatrice (mode : GWVA_MODE_SEND_GEM_MESSAGE_APNAME)
; in  : d0.w = taille du buffer  envoyer
; in  : d1.w = mode d'activation
;		GWVA_MODE_SEND_GEM_MESSAGE_APID
;		GWVA_MODE_SEND_GEM_MESSAGE_TO_MYSELF
;		GWVA_MODE_SEND_GEM_MESSAGE_APNAME
; in  : d2.w = apid de l'application destinatrice si mode GWVA_MODE_SEND_GEM_MESSAGE_APID
; out : d7.w = GWVA_NO_ERROR_GENERIC si pas d'erreur,
;	   GWVA_ERROR_ROOT_APNAME_NOT_FOUND si l'application n'a pas t trouve (GWVA_MODE_SEND_GEM_MESSAGE_APNAME)
;	   GWVA_ERROR_ROOT_APPL_WRITE_FAILED
;
; 08/03/98 : Cration
;------------------------------------------------------------------------------
GWVA_ROOT_SEND_GEM_MESSAGE:
	move.w global+_AESapid,bmsg_apid(a0)

	cmp.w #GWVA_MODE_SEND_GEM_MESSAGE_APNAME,d1
	bne.s .pas_par_apname
	save.l d0-d1/a0
	appl_find a1
	move.w d0,d2	; apid de l'application destinatrice
	load.l d0-d1/a0
	cmp.w #-1,d2
	beq.s .application_not_found
.pas_par_apname:

	cmp.w #GWVA_MODE_SEND_GEM_MESSAGE_TO_MYSELF,d1
	bne.s .pas_to_myself
	move.w global+_AESapid,d2
.pas_to_myself:

; On patch la longueur du buffer pour tre un multiple de 16 octets

	ifd PATCH_GWVA_ROOT_GEM_MESSAGE_PADDED16
	move.w d0,d3
	and.w #$f,d3
	beq.s .multiple16
	lsr.w #4,d0
	lsl.w #4,d0
	add.w #16,d0
.multiple16:
	endc	; ifd PATCH_GWVA_ROOT_GEM_MESSAGE_PADDED16

	move d0,d3
	sub.w #16,d3	; longueur du buffer tendu
	move.w d3,bmsg_extd(a0)

	appl_write d2,d0,a0
	tst.w d0
	beq.s .erreur_emission

	move.w #GWVA_NO_ERROR_GENERIC,d7
	rts

.erreur_emission:
	move.w #GWVA_ERROR_ROOT_APPL_WRITE_FAILED,d7
	rts
.application_not_found:
	move.w #GWVA_ERROR_ROOT_APNAME_NOT_FOUND,d7
	rts

 BSS
GWVA_ROOT_TMP_MSG_OUT_BUF:	ds.b L_BMSG
GWVA_ROOT_TMP_MSG_OUT_EXT_BUF:	ds.b 256
 TEXT

;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_ACTIVATE_METHODE_VIA_MESSAGE
; Description : Appel de la mthode d'un objet par message
;
; Rq  : S'applique uniquement aux "objets" avec handle V.A et aux "mthodes"
; Rq  : pas besoin du mode d'envoi (AP_NAME,AP_ID,ou MYSELF) car le routage se fait
; Rq  : par le No de handle
;
; in  : a0.l = pointeur sur la structure de l'objet dsirant envoyer le msg
; in  : d0-d2.l+a1-a2.l = registres  passer en paramtre
; in  : d3.w = handle de l'objet destinataire
; in  : d4.l = mthode de l'objet  activer
; in  : d5.w = masque des registres  envoyer : bit 0 = d0, bit 1 = d1, bit 2 = d2, bit 9 = a1, bit 10 = a2
; out : d7.w = GWVA_NO_ERROR_GENERIC si pas d'erreur,
;	   GWVA_ERROR_ROOT_APNAME_NOT_FOUND si l'application n'a pas t trouve (GWVA_MODE_SEND_GEM_MESSAGE_APNAME)
;	   GWVA_ERROR_ROOT_APPL_WRITE_FAILED
;
; 01/11/98 : Cration
;------------------------------------------------------------------------------
GWVA_ROOT_ACTIVATE_METHODE_VIA_MESSAGE:

	ifd OPT_GWVA_DEBUG_ERROR
	btst #GWVA_METHOD_BIT_EVENT,d4
	bne.s .error
	btst #GWVA_METHOD_BIT_BEFORE,d4
	bne.s .error
	btst #GWVA_METHOD_BIT_AFTER,d4
	bne.s .error
	endc	; ifd OPT_GWVA_DEBUG_ERROR

	lea GWVA_ROOT_TMP_MSG_OUT_BUF,a6
	move #GWVA_MSG_META_MESSAGE,bmsg_msgid(a6)
	move.w GWVA_OBJ_HANDLE(a0),bmsg_start_free_data(a6)	; handle orig
	move.w d3,bmsg_start_free_data+2(a6)		; handle dest
	move.l d4,bmsg_start_free_data+4(a6)		; mthode
	clr.w d6
	lea GWVA_ROOT_TMP_MSG_OUT_EXT_BUF,a5
	btst #0,d5
	beq.s .pas_d0
	move.l d0,(a5)+
	addq #1,d6
.pas_d0:
*	movem.l d0-d2/a1-a2,GWVA_ROOT_TMP_MSG_OUT_EXT_BUF
	move.w d6,bmsg_start_free_data+8(a6)		; nombre de registre .l
	moveq #L_BMSG,d0				; taille : 16+ 5 registres.l
	lsl #2,d6					; pour un mot long
	add.w d6,d0
	move #GWVA_MODE_SEND_GEM_MESSAGE_TO_MYSELF,d1		; mode

	save.l a0
	move.l a6,a0
	bsr GWVA_ROOT_SEND_GEM_MESSAGE
	load.l a0
	rts

	ifd OPT_GWVA_DEBUG_ERROR
.error:	GWVA_PROGRAMMER_ERROR	GWVA_ASCII_ERROR_NO_EVENT_PIPED
	moveq #GWVA_NO_ERROR_GENERIC,d7
	rts

	endc	; ifd OPT_GWVA_DEBUG_ERROR

;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_KILL_APPLICATION
; Description : Permet de sortir de l'application, sans effet pour les accessoires.
;
; Rq  : Envoie une mthode DELETE sur l'objet WAPP
; Rq  : Pour sortir de l'application faire un bsr  cette fonction
;
; in  : -
; out : -
;
; 11/11/98 : Cration
;------------------------------------------------------------------------------
GWVA_ROOT_KILL_APPLICATION:

	tst GWVA_ROOT_IS_ACC_PRG
	beq.s .acc_dont_go_out

	move.l GWVA_ROOT_PTR_WAPP,a0		; sender
	move.w GWVA_OBJ_HANDLE(a0),d3		; handle dest
	move.l #GWVA_METHOD_DELETE,d4		; mthode

	moveq #GWVA_NO_ERROR_GENERIC,d0		; paramtre entre mth DELETE
	move.w #%1,d5			; paramtre d0

	bra GWVA_ROOT_ACTIVATE_METHODE_VIA_MESSAGE
	*rts

.acc_dont_go_out:
	rts

;------------------------------------------------------------------------------

	ifd MDL_COLORS

;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_SET_FALCON_PALETTE
; Description : Utilise la palette au format RVB Falcon
;
; Rq  : Le format de chaque couleur est dummy.b R.b V.b B.b
;
; in  : a0.l = pointeur sur un buffer contenant la palette
; out : -
;
; 14/03/98 : Cration
;------------------------------------------------------------------------------
GWVA_ROOT_SET_FALCON_PALETTE:

	moveq #1,d0		; est-on dans une rsolution avec palette ?
	move.w GWVA_ROOT_VDI_WORKSTATION_EXTD+4*2,d1
	lsl.w d1,d0
	cmp.w GWVA_ROOT_VDI_WORKSTATION+13*2,d0
	bne.s .pas_resolution_palette

	move.l a0,-(sp)
	move GWVA_ROOT_VDI_WORKSTATION+13*2,-(sp)
	clr -(sp)
	appel xbios,vsetrgb,10		; QUE FALCON et COMPATIBLE

.pas_resolution_palette:
	rts

;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_SET_VDI_PALETTE
; Description : Utilise la palette au format RVB VDI
;
; Rq  : Le format de chaque couleur est R.w V.w B.w en pour 1000
;
; in  : a0.l = pointeur sur un buffer contenant la palette
; out : -
;
; 14/03/98 : Cration
;------------------------------------------------------------------------------
GWVA_ROOT_SET_VDI_PALETTE:
	bftst ([GWVA_ROOT_PTR_WAPP.l],GWVA_WIND_OBJ_STATUS){GWVA_WIND_OLBIT_OPEN:1}
	beq.s .not_open

	moveq #1,d0
	move.w GWVA_ROOT_VDI_WORKSTATION_EXTD+4*2,d1
	lsl.w d1,d0
	cmp.w GWVA_ROOT_VDI_WORKSTATION+13*2,d0
	bne.s .pas_de_palette

	moveq #0,d0
.definit_palette_systeme:
	save.l d0/a0
	vs_color d0,(a0),2(a0),4(a0)
	load.l d0/a0
	lea 3*2(a0),a0
	addq.w #1,d0
	cmp.w GWVA_ROOT_VDI_WORKSTATION+13*2,d0
	bne.s .definit_palette_systeme

.pas_de_palette:
.not_open:
	rts

;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_STORE_VDI_PALETTE
; Description : Sauvegarde la palette au format VDI
;
; Rq  : Le format de chaque couleur est R.w V.w B.w en pour 1000
;
; in  : a0.l = pointeur sur un buffer
; out : -
;
; 14/03/98 : Cration
;------------------------------------------------------------------------------
GWVA_ROOT_STORE_VDI_PALETTE:
	bftst ([GWVA_ROOT_PTR_WAPP.l],GWVA_WIND_OBJ_STATUS){GWVA_WIND_OLBIT_OPEN:1}
	beq.s .not_open

	moveq #1,d0
	move GWVA_ROOT_VDI_WORKSTATION_EXTD+4*2,d1
	lsl.w d1,d0
	cmp.w GWVA_ROOT_VDI_WORKSTATION+13*2,d0
	bne.s .pas_de_palette

	moveq #0,d0
.recopie_palette_systeme:
	save.l d0/a0
	vq_color d0,#COLOR_ACTUAL
	load.l d0/a0
	move.w intout+1*2,(a0)+
	move.w intout+2*2,(a0)+
	move.w intout+3*2,(a0)+
	addq.w #1,d0
	cmp.w GWVA_ROOT_VDI_WORKSTATION+13*2,d0
	bne.s .recopie_palette_systeme

.pas_de_palette:
.not_open:
	rts

;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_SET_DESK_PALETTE
; Description : Restaure la palette systme
;
; Rq  : La palette n'est initialis que si l'APP  t ouvert.
; Rq  : Sauvegarde a0.l
;
; in  : -
; out : -
;
; 14/03/98 : Cration
;------------------------------------------------------------------------------
GWVA_ROOT_SET_DESK_PALETTE:
	save.l a0
	lea GWVA_ROOT_DESK_PALETTE,a0
	bsr GWVA_ROOT_SET_VDI_PALETTE
	load.l a0
	rts

	endc	; ifd MDL_COLORS

	endc	; ifd MDL_GEM

;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_TEST_IF_OBJECT_CREATED
; Description : Test si l'objet est cr
;
; Rq  : ne touche aucun registre !
;
; in  : a0.l = pointeur sur l'objet
; out : d7.w = GWVA_NO_ERROR_GENERIC si l'objet existe,
;	   GWVA_ERROR_GENERIC sinon.
;
; 27/02/98 : Cration
;------------------------------------------------------------------------------
GWVA_ROOT_TEST_IF_OBJECT_CREATED:
	cmp.w #GWVA_OBJ_CTE_HANDLE_UNDEF,GWVA_OBJ_HANDLE(a0)
	beq.s .no_exist
	moveq #GWVA_NO_ERROR_GENERIC,d7
	rts
.no_exist:	moveq #GWVA_ERROR_GENERIC,d7
	rts

;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_CREATE_OBJECT
; Description : init mth par dfaut,cre le handle puis appel CREATE de l'objet
;
; in  : a0.l = pointeur sur l'objet  crer
; out : d7.w = GWVA_NO_ERROR_GENERIC ou erreur
;	  GWVA_ERROR_ROOT_HANDLE_ALREADY_ALLOCATED
;	  GWVA_ERROR_ROOT_INIT_METHOD_LIST
;	  + erreurs eventuelles de la mthode CREATE de l'objet
;
; 31/01/98 : Cration
;------------------------------------------------------------------------------
GWVA_ROOT_CREATE_OBJECT:

*	bsr GWVA_ROOT_PRG_INIT_DEFAULT_METHODLIST_PTR ; A transformer en vrification d'existence de classe
*	cmp.w #GWVA_NO_ERROR_GENERIC,d7
*	bne.s .error_init_default_list

	bsr GWVA_ROOT_PRG_ALLOCATE_HANDLE
	cmp.w #GWVA_NO_ERROR_GENERIC,d7
	bne.s .error_handle_already_allocated

	GWVA_CALL_METHOD_OTHER_OBJ	#GWVA_METHOD_CREATE
	cmp.w #GWVA_NO_ERROR_GENERIC,d7
	beq.s .no_error

	save.w d6/d7
	GWVA_CALL_METHOD_OTHER_OBJ	#GWVA_METHOD_DELETE
	load.w d6/d7

.no_error:
.error_handle_already_allocated:
.error_init_default_list:
	rts

	ifd MDL_EVENTS

;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_RETURN_ITEMLIST_PTR_FROM_KC
; Description : retourne un pointeur sur le dbut d'une struct contenant le shortcut
;
; in  : a0.l = adresse 1er objet dans la liste  parcourir
; in  : d0.w = Key code
; in  : d1.w = Key state
; in  : d2.w = longueur de la structure d'un item
; out : a0.l = pointeur sur le dbut structure de l'item contenant le shortcut
; out : d7.w = GWVA_ERROR_GENERIC ou GWVA_NO_ERROR_GENERIC
;
; 25/01/98 : Cration
;------------------------------------------------------------------------------

*	ifd MDL_MENU
*GWVA_MENU_OR_WIND_FORM
*	else
*		ifd MDL_WIND_FORM
*GWVA_MENU_OR_WIND_FORM
*		endc
*	endc
*
*	ifd GWVA_MENU_OR_WIND_FORM

GWVA_ROOT_RETURN_ITEMLIST_PTR_FROM_KC:
; d3.w : masque pour les touches spciales  prendre en compte dans les comparaisons

	move.b #K_ALT,d3	; seule la touche alternate au dbut

; Donc si l'ikbd.bas est non nul, on se fout du shift car il est
; dj compris dans le code : ex. ctrl+4=shift/ctrl+' <> ctrl+'
;
; Donc vitez ctrl ou alt + touches qui varient avec le shift selon les pays, autres que A-Z
; Pb : ^U et ^U donne le mme code $1615 : il faut tout de mme vrifier les shifts, mais que
; dans le cas des lettres A-Z

	tst.b d0
	bne.s .search_in_table

	save.l d0-d2/a0
	pea -1
	pea -1
	pea -1
	appel xbios,keytbl,14	; rcupre la table de conversion SCAN_CODE->ASCII
			; dans l'ordre : unshifte/shifte/caps-locke
	move.l d0,a1
	load.l d0-d2/a0

	move.l (a1),a2	; table unshifte

	btst #BIT_K_RSHIFT,d1
	bne.s .shift
	btst #BIT_K_LSHIFT,d1
	beq.s .pas_shift
.shift:	
	move.l 4(a1),a2	; table shifte
.pas_shift:
	lsr.w #8,d0		; on s'indexe grace au scan code

	tst.b (a2,d0.w)
	beq.s .pas_ascii
	move.b (a2,d0.w),d0	; d0 = ascii du scan-code pour le pays en cours
	move.b #K_CTRL|K_ALT,d3	; le masque : on ne doit pas tenir compte des shifts pour les comparaisons
	bra.s .search_in_table
.pas_ascii:	bset #7,d0
	move.b #K_RSHIFT|K_LSHIFT|K_CTRL|K_ALT,d3	; On doit avoir une identit exacte entre touche/state avec la liste
*	bra.s .search_in_table

.search_in_table:
	cmp.w #GWVA_ROOT_CTE_SHORTCUT_ENDS_LIST,GWVA_COMMON_LISTOBJ_SHORTCUT(a0)
	beq.s .finis

	cmp.b GWVA_COMMON_LISTOBJ_SHORTCUT+1(a0),d0
	beq.s .compare_touches_speciales

.loop:	lea (a0,d2.w),a0
	bra.s .search_in_table

.compare_touches_speciales:
	move.b GWVA_COMMON_LISTOBJ_SHORTCUT(a0),d4	; touches spciales

	cmp.b #'a'-'a'+1,d0
	blt.s .pas_touche_control_alphabtique
	cmp.b #'z'-'a'+1,d0
	bgt.s .pas_touche_control_alphabtique

; On doit tenir compte des shifts dans le cas d'une touche alphabtique conjugue
; avec la touche control : ^U et ^U donne le mme code $1615

	or.b #K_RSHIFT|K_LSHIFT,d3

.pas_touche_control_alphabtique:

	move.b d4,d6
	and.b #K_RSHIFT|K_LSHIFT,d6		; test des shifts
	beq.s .pas_de_shift_voulue
	move.b d1,d6
	and.b #K_RSHIFT|K_LSHIFT,d6
	beq.s .pas_de_shift_propos
	and.b #~(K_RSHIFT|K_LSHIFT),d3	; si tous le monde veut les shifts, on n'en tient pas compte dans le masque
				; ce qui entraine que le shift gauche est identique au shift droit
.pas_de_shift_propos:
.pas_de_shift_voulue:

	move.b d1,d6	; touches spciales proposes
	and.b d3,d6		; On masque les bits qui ne nous interressent plus
	and.b d3,d4		; De mme sur les touches spciales voulues

	cmp.b d4,d6
	bne.s .loop
	moveq #GWVA_NO_ERROR_GENERIC,d7		; a0 pointe bien
	rts
.finis:	moveq #GWVA_ERROR_GENERIC,d7
	rts

*	endc	; ifd GWVA_MENU_OR_WIND_FORM

;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_PRG_CLEAR_EVNT_BIT
; Description : efface un vnement dans la mmoire vnementielle du evnt_multi V.A
;
; in  : d0.w = numro du bit d'event
;
; 24/01/98 : Cration
;------------------------------------------------------------------------------
GWVA_ROOT_PRG_CLEAR_EVNT_BIT:
	move.w GWVA_ROOT_EVENT_BITS,d1
	bclr d0,d1
	move.w d1,GWVA_ROOT_EVENT_BITS
	rts

	endc	; ifd MDL_EVENTS

;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_PRG_ALLOCATE_HANDLE
; Description : Allocation d'un handle unique  l'objet
;
; in  : a0.l = pointeur sur l'objet
; out : d7.w = GWVA_NO_ERROR_GENERIC ou GWVA_ERROR_ROOT_HANDLE_ALREADY_ALLOCATED
;
; 10/01/98 : Cration
;------------------------------------------------------------------------------
GWVA_ROOT_PRG_ALLOCATE_HANDLE:
	cmp.w #GWVA_OBJ_CTE_HANDLE_UNDEF,GWVA_OBJ_HANDLE(a0)
	bne.s .deja_cree

	move.w GWVA_ROOT_HANDLE_COUNTER,GWVA_OBJ_HANDLE(a0)
	addq.w #1,GWVA_ROOT_HANDLE_COUNTER

	moveq #GWVA_NO_ERROR_GENERIC,d7
	rts
.deja_cree:	move.w #GWVA_ERROR_ROOT_HANDLE_ALREADY_ALLOCATED,d7
	rts

;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_PRG_STORE_CURRENT_PATH
; Description : sauve le PATH courant
;
; in  : a0.l = pointeur o stocker le path courant (U:\PATH\)
;
; 11/01/98 : Cration
;------------------------------------------------------------------------------
GWVA_ROOT_PRG_STORE_CURRENT_PATH:

	save.l a0
	DGETDRV d0
	load.l a0
	add.b #'A',d0
	move.b d0,(a0)+
	move.b #':',(a0)+

	clr.b (a0)
	save.l a0
	DGETPATH #0,(a0)
	load.l a0

.find_end:	tst.b (a0)+
	bne.s .find_end
	move.w #'\'*256,-1(a0)
	rts

;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_PRG_SEARCH_SYSTEM_VARIABLES_AND_COOKIES
; Description : cherche un certain nombre de cookies et variables environnement
;
; Rq  : Fonction  excuter en superviseur (SUPEXEC)
;
; 11/01/98 : Cration
;------------------------------------------------------------------------------
GWVA_ROOT_PRG_SEARCH_SYSTEM_VARIABLES_AND_COOKIES:

	move.l _sysbase.w,a0		; Adresse de la ROM
	move.w 2(a0),GWVA_ROOT_SYSVAR_TOS_VERSION

	move.w _bootdev.w,GWVA_ROOT_SYSVAR_BOOT_DEVICE

	ifd LIB_SEARCH_SYSTEM_COOKIES

	move.l #'FSEL',GWVA_ROOT_SEARCH_IN_OUT
	bsr GWVA_COOKIE_SEARCH_ONE_COOKIE
	move.l GWVA_ROOT_SEARCH_IN_OUT,a0
	cmp.l #0,a0
	beq.s .no_fsel
	move.l (a0),a1
	cmp.l #'SLCT',(a1)
	bne.s .no_trix
	move.l a1,GWVA_ROOT_COOKIE_SELECTRIX_PTR
	lea 4+2+2+2+2+4+2+4(a1),a1
	move.l a1,GWVA_ROOT_COOKIE_SELECTRIX_COMM_WORD
	lea 2+2+4(a1),a1
	move.l a1,GWVA_ROOT_COOKIE_SELECTRIX_OUT_COUNT
	lea 2(a1),a1
	move.l a1,GWVA_ROOT_COOKIE_SELECTRIX_OUT_PTR
.no_fsel:
.no_trix:

	move.l #"_FPU",GWVA_ROOT_SEARCH_IN_OUT
	bsr GWVA_COOKIE_SEARCH_ONE_COOKIE
	tst.l GWVA_ROOT_SEARCH_IN_OUT
	beq.s .no_copro
	move.w ([GWVA_ROOT_SEARCH_IN_OUT]),GWVA_ROOT_COOKIE_COPRO
.no_copro:

	move.l #"MiNT",GWVA_ROOT_SEARCH_IN_OUT
	bsr GWVA_COOKIE_SEARCH_ONE_COOKIE
	tst.l GWVA_ROOT_SEARCH_IN_OUT
	beq.s .no_mint
	move.w (2,[GWVA_ROOT_SEARCH_IN_OUT]),GWVA_ROOT_COOKIE_MINT
.no_mint:

	move.l #"MagX",GWVA_ROOT_SEARCH_IN_OUT	; problme avec la relocation de ressource
	bsr GWVA_COOKIE_SEARCH_ONE_COOKIE
	tst.l GWVA_ROOT_SEARCH_IN_OUT
	beq.s .no_magic
	move.l ([GWVA_ROOT_SEARCH_IN_OUT]),GWVA_ROOT_COOKIE_MAGIC
.no_magic:

	move.l #"Gnva",GWVA_ROOT_SEARCH_IN_OUT
	bsr GWVA_COOKIE_SEARCH_ONE_COOKIE
	tst.l GWVA_ROOT_SEARCH_IN_OUT
	beq.s .no_geneva
	move.l ([GWVA_ROOT_SEARCH_IN_OUT]),GWVA_ROOT_COOKIE_GENEVA
.no_geneva:

	endc	; ifd LIB_SEARCH_SYSTEM_COOKIES

	rts

	ifd MDL_GEM

;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_PRG_INFO_VDI_WORKSTATION
; Description : rcupre toutes les infos sur la station VDI ouverte
;
; Rq  : stockage dans GWVA_ROOT_VDI_WORKSTATION + GWVA_ROOT_VDI_WORKSTATION_EXTD
; Rq  : rcupre aussi	GWVA_ROOT_SYSTEM_LARGEFONT_HEIGHT / GWVA_ROOT_SYSTEM_SMALLFONT_HEIGHT
; Rq  :		GWVA_ROOT_TEXT_WIDTH et suivants
;
; 10/01/98 : Cration
;------------------------------------------------------------------------------
GWVA_ROOT_PRG_INFO_VDI_WORKSTATION:
	vq_extnd #0		; on demande les info VDI de la station courante

	lea intout,a0
	lea GWVA_ROOT_VDI_WORKSTATION,a1
	move.w #45-1,d0
.recopy_vdi_parameters1:
	move.w (a0)+,(a1)+
	dbf d0,.recopy_vdi_parameters1

	lea ptsout,a0
	move.w #12-1,d0
.recopy_vdi_parameters2:
	move.w (a0)+,(a1)+
	dbf d0,.recopy_vdi_parameters2

	vq_extnd #1

	lea intout,a0
	lea GWVA_ROOT_VDI_WORKSTATION_EXTD,a1
	move.w #45-1,d0
.recopy_vdi_parameters3:
	move.w (a0)+,(a1)+
	dbf d0,.recopy_vdi_parameters3

	lea ptsout,a0
	move.w #12-1,d0
.recopy_vdi_parameters4:
	move.w (a0)+,(a1)+
	dbf d0,.recopy_vdi_parameters4

	vqt_attributes
	movem.w ptsout,d0-d3	; -> largeur/hauteur caractre et cellule
	movem.w d0-d3,GWVA_ROOT_TEXT_WIDTH
; d1.w : character height = taille par dfaut de la police systme  l'ouverture de
; la station virtuelle.
	move.w d1,GWVA_ROOT_SYSTEM_LARGEFONT_HEIGHT	; par dfaut
	move.w #4,GWVA_ROOT_SYSTEM_SMALLFONT_HEIGHT	; taille des polices systmes (petite et normale)

	cmp.w #$400,global+_AESversion
	blt.s .pas_de_appl_getinfo

	appl_getinfo #0
	move.w int_out+1*2,GWVA_ROOT_SYSTEM_LARGEFONT_HEIGHT

	appl_getinfo #1
	move.w int_out+1*2,GWVA_ROOT_SYSTEM_SMALLFONT_HEIGHT

.pas_de_appl_getinfo:

	rts

;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_PRG_ALERT_BOX
; Description : Fonction d'affichage d'une boite d'alerte AES
;
; Rq  : Prend en compte l'ventuel prsence d'une fentre BITMAP (changement
; Rq  : de la palette couleur).
;
; in  : a0.l = pointeur sur la chaine d'alerte
; in  : d0.w = numro du bouton par dfaut gauche(1)->milieu(2)->droite(3)
; out : d0.w = numro du bouton press
;
; 11/01/98 : Cration
; 14/03/98 : Modification pour les palettes de couleurs
;------------------------------------------------------------------------------
GWVA_ROOT_PRG_ALERT_BOX:

	ifd MDL_COLORS
	save.l d0/a0
	lea GWVA_ROOT_TMP_PALETTE,a0
	bsr GWVA_ROOT_STORE_VDI_PALETTE
	bsr GWVA_ROOT_SET_DESK_PALETTE
	load.l d0/a0

	endc	; ifd MDL_COLORS

	form_alert d0,a0
	move.w int_out,d0

	ifd MDL_COLORS

	save.w d0
	lea GWVA_ROOT_TMP_PALETTE,a0
	bsr GWVA_ROOT_SET_VDI_PALETTE
	load.w d0

	endc	; ifd MDL_COLORS

	rts

	endc	; ifd MDL_GEM

	ifd MDL_WIND_MENU

;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_PRG_MENU_FIX_OBJECT_ARROW
; Description : fixe en mmoire les 2 flches pour menus dans les fentres
;
; 11/01/98 : Cration
;------------------------------------------------------------------------------
GWVA_ROOT_PRG_MENU_FIX_OBJECT_ARROW:
	rsrc_obfix #GWVA_ROOT_MENU_TREE_ARROWS,#0
	rsrc_obfix #GWVA_ROOT_MENU_TREE_ARROWS,#1
	rsrc_obfix #GWVA_ROOT_MENU_TREE_ARROWS,#2
	rts	

	endc	; ifd MDL_WIND_MENU

	ifd MDL_EVENTS

	ifne GWVA_ROOT_CTE_MAX_TIMER

;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_ELABORATE_TIMER_AND_EXEC_METHOD
; Description : calcule le timer pour la boucle GEM et excute les mthodes si besoin
;
; Rq  : -
;
; out : GWVA_ROOT_TIMER_COUNT.l
;
; 15/01/98 : Cration
;------------------------------------------------------------------------------
GWVA_ROOT_ELABORATE_TIMER_AND_EXEC_METHOD:

	SUPEXEC GWVA_200HZ_GET_CURRENT_COUNTER
	move.l GWVA_200HZ_CURRENT_COUNTER,d0
	; on le prend au dbut pour viter de boucler

	lea GWVA_ROOT_TIMER_LIST,a1
	move.l #999999,d2

	move.w #GWVA_ROOT_CTE_MAX_TIMER-1,d1
.parse_all_timers:
	tst.l GWVA_ROOT_OBJ_TIMER_OBJ_PTR(a1)
	beq.s .none_here

.comute_delay:
	move.l d0,d3
	sub.l GWVA_ROOT_OBJ_TIMER_LAST_COUNT_200(a1),d3
	mulu.l #1000/200,d3
	move.l GWVA_ROOT_OBJ_TIMER_COUNT(a1),d4

	cmp.l d3,d4
	bgt.s .time_not_elapsed

	; d0/d1/d2/a1 vont tre sauvs
	move.l GWVA_ROOT_OBJ_TIMER_OBJ_PTR(a1),a0
	GWVA_CALL_METHOD_OTHER_OBJ	GWVA_ROOT_OBJ_TIMER_METHOD(a1)

	bftst GWVA_ROOT_OBJ_TIMER_PROPERTIES(a1){GWVA_ROOT_OLBIT_TIMER_REARM:1}
	beq.s .pas_re_arm

	move.l d0,GWVA_ROOT_OBJ_TIMER_LAST_COUNT_200(a1)
	bra.s .comute_delay

.pas_re_arm:clr.l GWVA_ROOT_OBJ_TIMER_OBJ_PTR(a1)	; ne pas calculer son dlai

.none_here:
.continue_research:
	lea GWVA_ROOT_OBJ_TIMER_STRUCT_LENGTH(a1),a1
	dbf d1,.parse_all_timers

	clr.l GWVA_ROOT_TIMER_COUNT
	cmp.l #999999,d2
	beq.s .pas_de_timer
	move.l d2,GWVA_ROOT_TIMER_COUNT
.pas_de_timer:
	rts

.time_not_elapsed:
	sub.l d3,d4		; temps qu'il reste  passer avant le prochain
	divu.l #1000/200,d4

	tst.l d4
	bne.s .pas_nul

	moveq #1,d4
.pas_nul:
	cmp.l d2,d4
	bge.s .pas_plus_petit_que_la_reference

	move.l d4,d2

.pas_plus_petit_que_la_reference
	bra.s .continue_research

;------------------------------------------------------------------------------

	endc	; ifne GWVA_ROOT_CTE_MAX_TIMER


	ifge GWVA_ROOT_CTE_MAX_MOUSEZONE-1

;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_ELABORATE_MOUSE_ZONES
; Description : Calcul les deux mouse zones en fonction des zones enregistres
;
; Rq  : La mouse zone 1 est rserve aux entres de zone, la mouse zone 2 au
; Rq  : sorties
;
; out : GWVA_ROOT_M[12]_ACTIVATED, GWVA_ROOT_M[12]FLAG, GWVA_ROOT_M[12][XYWH]
;
; 28/03/98 : Cration
;------------------------------------------------------------------------------
GWVA_ROOT_ELABORATE_MOUSE_ZONES:

; on doit en premier dterminer la position de la souris

	graf_mkstate

	move.w int_out+1*2,d0	; x
	move.w int_out+2*2,d1	; y

; on va dterminer la zone de sortie :
; on va chercher dans combien de zone de sortie la souris se trouve :
;	- si elle est dans aucune zone mais que des zones existent, 
;	  on prend n'importe laquelle -> avertissement immdiat
;	- si elle est dans une zone : on prend cette zone
;	- si elle est dans plus d'une zone, on effectue l'intersection
;	  de toutes ces zones -> zone dfinie

	clr.w GWVA_ROOT_M2_ACTIVATED
	move.w #MO_LEAVE,GWVA_ROOT_M2FLAG

	clr.l GWVA_ROOT_M2W

	lea GWVA_ROOT_MOUSEZONE_LIST,a1
	move.w #GWVA_ROOT_CTE_MAX_MOUSEZONE-1,d6

.parse_all_leave_mouse_zone:
	tst.l GWVA_ROOT_OBJ_MOUSEZONE_OBJ_PTR(a1)
	beq.s .none_here

	cmp.w #MO_LEAVE,GWVA_ROOT_OBJ_MOUSEZONE_FLAG(a1)
	bne.s .pas_zone_sortie

	move.w #1,GWVA_ROOT_M2_ACTIVATED	; au moins une zone dfinie

	tst.l GWVA_ROOT_M2W
	bne.s .test_zone
	move.l GWVA_ROOT_OBJ_MOUSEZONE_X(a1),GWVA_ROOT_M2X
	move.l GWVA_ROOT_OBJ_MOUSEZONE_W(a1),GWVA_ROOT_M2W
	bra.s .pas_test_zone
.test_zone:

	lea GWVA_ROOT_OBJ_MOUSEZONE_X(a1),a0
	bsr GWVA_ROOT_TEST_XY_IN_XYWH
	cmp.w #GWVA_NO_ERROR_GENERIC,d7
	bne.s .pas_dans_zone

	save.l d0-d1/d6/a1
	lea GWVA_ROOT_M2X,a1
	bsr GWVA_ROOT_INTERSEC_ZONES
	load.l d0-d1/d6/a1

.pas_test_zone:
.pas_dans_zone:
.pas_zone_sortie:
.none_here:
	lea GWVA_ROOT_OBJ_MOUSEZONE_STRUCT_LENGTH(a1),a1
	dbf d6,.parse_all_leave_mouse_zone

; on va dterminer la zone d'entre :
; on va chercher si on est dans aucune zone
;	- si une seule zone dfinie -> on prend cette zone en entr
;	- si plusieurs zone :
;	  xmin = max(x) < xmouse ou xmin ecran
;	  xmax = min(x) > xmouse ou xmax ecran
;	  ymin = max(y) < ymouse ou ymin ecran
;	  ymax = min(y) > ymouse ou ymax ecran
;	  -> zone definit en SORTIE

	clr.w GWVA_ROOT_M1_ACTIVATED

	save.w d0-d1
	move.l GWVA_ROOT_PTR_WAPP,a0
	bsr GWVA_WIND_RETURN_INT_ABS_POS_SIZE
	movem.w d0-d3,GWVA_ROOT_M1X
	load.w d0-d1

	lea GWVA_ROOT_MOUSEZONE_LIST,a1
	move.w #GWVA_ROOT_CTE_MAX_MOUSEZONE-1,d6

.parse_all_enter_mouse_zone:
	tst.l GWVA_ROOT_OBJ_MOUSEZONE_OBJ_PTR(a1)
	beq .none_here2

	cmp.w #MO_ENTER,GWVA_ROOT_OBJ_MOUSEZONE_FLAG(a1)
	bne .pas_zone_entree

	move.w #1,GWVA_ROOT_M1_ACTIVATED

	lea GWVA_ROOT_OBJ_MOUSEZONE_X(a1),a0
	bsr GWVA_ROOT_TEST_XY_IN_XYWH
	cmp.w #GWVA_NO_ERROR_GENERIC,d7
	beq .dans_zone

	move.w #MO_LEAVE,GWVA_ROOT_M1FLAG

; d0/d1 = x,y souris

	move.w GWVA_ROOT_OBJ_MOUSEZONE_X(a1),d2	; bord gauche
	move.w GWVA_ROOT_M1X,d4
	add.w GWVA_ROOT_M1W,d4
	sub.w #1,d4				; bord droit de la zone
	cmp.w d2,d0
	bgt.s .souris_a_droite_bord_gauche
	cmp.w d4,d2
	bge.s .supx
	add.w #1,d2
	sub.w GWVA_ROOT_M1X,d2
	move.w d2,GWVA_ROOT_M1W			; nouveau bord  droit de la zone
.supx:
.souris_a_droite_bord_gauche:

	move.w GWVA_ROOT_OBJ_MOUSEZONE_Y(a1),d3	; bord haut
	move.w GWVA_ROOT_M1Y,d5			; bord bas de la zone
	add.w GWVA_ROOT_M1H,d5
	sub.w #1,d5

	cmp.w d3,d1
	bgt.s .souris_en_dessous_bord_haut
	cmp.w d5,d3
	bge.s .supy
	add.w #1,d3
	sub.w GWVA_ROOT_M1Y,d3
	move.w d3,GWVA_ROOT_M1H
.supy:
.souris_en_dessous_bord_haut:

	move.w GWVA_ROOT_OBJ_MOUSEZONE_X(a1),d2
	add.w GWVA_ROOT_OBJ_MOUSEZONE_W(a1),d2
	sub.w #1,d2				; bord droit
	move.w GWVA_ROOT_M1X,d4			; bord haut zone
	cmp.w d2,d0
	blt.s .souris_a_gauche_bord_droit
	cmp.w d4,d2
	ble.s .infx
	move.w d2,GWVA_ROOT_M1X
.infx:
.souris_a_gauche_bord_droit:

	move.w GWVA_ROOT_OBJ_MOUSEZONE_Y(a1),d3
	add.w GWVA_ROOT_OBJ_MOUSEZONE_H(a1),d3
	sub.w #1,d3				; bord bas
	move.w GWVA_ROOT_M1Y,d5			; bord bas de la zone
	cmp.w d3,d1
	blt.s .souris_au_dessus_bord_bas
	cmp.w d5,d3
	ble.s .infy
	move.w d3,GWVA_ROOT_M1Y
.infy:
.souris_au_dessus_bord_bas

.pas_zone_entree:
.none_here2:
	lea GWVA_ROOT_OBJ_MOUSEZONE_STRUCT_LENGTH(a1),a1
	dbf d6,.parse_all_enter_mouse_zone
.fin_parsing_zone_entree:

	rts

.dans_zone:	; si au moins dans une zone, on prend une de ces zones en entre
	move.l GWVA_ROOT_OBJ_MOUSEZONE_X(a1),GWVA_ROOT_M1X
	move.l GWVA_ROOT_OBJ_MOUSEZONE_W(a1),GWVA_ROOT_M1W
	move.w #1,GWVA_ROOT_M1_ACTIVATED
	move.w #MO_ENTER,GWVA_ROOT_M1FLAG
	bra .fin_parsing_zone_entree

;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_EXECUTE_MOUSE_ZONES_METHODES
; Description : Execute les mthodes lies au mouse zones
;
; Rq  : La mouse zone 1 est rserve aux entres de zone, la mouse zone 2 au
; Rq  : sorties
; Rq  : La position x,y de la souris est passe  la mthode dans d0.w, d1.w
;
; in  : d0-d1.w = x,y de la souris
; in  : d2.w = mot contenant les bits d'activation vnementielle EVNT_MOUSE1 et EVNT_MOUSE2
;
; 29/03/98 : Cration
;------------------------------------------------------------------------------
GWVA_ROOT_EXECUTE_MOUSE_ZONES_METHODES:
	btst #EVNT_MOUSE1,d2
	bne.s .zone_entree

; zone de sortie = zone 2

	lea GWVA_ROOT_MOUSEZONE_LIST,a1
	move.w #GWVA_ROOT_CTE_MAX_MOUSEZONE-1,d6

.parse_all_leave_mouse_zone:
	tst.l GWVA_ROOT_OBJ_MOUSEZONE_OBJ_PTR(a1)
	beq.s .none_here

	cmp.w #MO_LEAVE,GWVA_ROOT_OBJ_MOUSEZONE_FLAG(a1)
	bne.s .pas_zone_sortie

	lea GWVA_ROOT_OBJ_MOUSEZONE_X(a1),a0
	bsr GWVA_ROOT_TEST_XY_IN_XYWH
	cmp.w #GWVA_NO_ERROR_GENERIC,d7
	beq.s .dans_zone

; Si une zone est dfinie en sortie et que la souris ne se trouve pas dedans,
; on doit alors activer la mthode voulue
; En entre de la mthode on a la position de la souris dans d0.w, d1.w

	move.l GWVA_ROOT_OBJ_MOUSEZONE_OBJ_PTR(a1),a0
	clr.l GWVA_ROOT_OBJ_MOUSEZONE_OBJ_PTR(a1)	; on la dsactive tout de suite, si la mthode la ractive ...

	save.w d0-d1
	bsr GWVA_ROOT_TEST_IF_OBJECT_CREATED
	load.w d0-d1
	cmp.w #GWVA_NO_ERROR_GENERIC,d7
	bne.s .objet_pas_cree1

	move.w GWVA_ROOT_EVNT_MULTI_KS,d2
	swap d2
	move.w GWVA_ROOT_EVNT_MULTI_MB,d2

	save.w d6
	GWVA_CALL_METHOD_OTHER_OBJ	GWVA_ROOT_OBJ_MOUSEZONE_METHOD(a1)
	load.w d6

.objet_pas_cree1:
.dans_zone:
.pas_zone_sortie:
.none_here:
	lea GWVA_ROOT_OBJ_MOUSEZONE_STRUCT_LENGTH(a1),a1
	dbf d6,.parse_all_leave_mouse_zone

	rts
.zone_entree:

	lea GWVA_ROOT_MOUSEZONE_LIST,a1
	move.w #GWVA_ROOT_CTE_MAX_MOUSEZONE-1,d6

.parse_all_enter_mouse_zone:
	tst.l GWVA_ROOT_OBJ_MOUSEZONE_OBJ_PTR(a1)
	beq.s .none_here2

	cmp.w #MO_ENTER,GWVA_ROOT_OBJ_MOUSEZONE_FLAG(a1)
	bne.s .pas_zone_entree

	lea GWVA_ROOT_OBJ_MOUSEZONE_X(a1),a0
	bsr GWVA_ROOT_TEST_XY_IN_XYWH
	cmp.w #GWVA_NO_ERROR_GENERIC,d7
	bne.s .pas_dans_zone

; Si une zone est dfinie en entre et que la souris se trouve dedans,
; on doit alors activer la mthode voulue.
; En entre de la mthode on a la position de la souris dans d0.w, d1.w

	move.l GWVA_ROOT_OBJ_MOUSEZONE_OBJ_PTR(a1),a0
	clr.l GWVA_ROOT_OBJ_MOUSEZONE_OBJ_PTR(a1)	; on la dsactive tout de suite, si la mthode la ractive ...

	save.w d0-d1
	bsr GWVA_ROOT_TEST_IF_OBJECT_CREATED
	load.w d0-d1
	cmp.w #GWVA_NO_ERROR_GENERIC,d7
	bne.s .objet_pas_cree2

	move.w GWVA_ROOT_EVNT_MULTI_KS,d2
	swap d2
	move.w GWVA_ROOT_EVNT_MULTI_MB,d2

	save.w d6
	GWVA_CALL_METHOD_OTHER_OBJ	GWVA_ROOT_OBJ_MOUSEZONE_METHOD(a1)
	load.w d6

.objet_pas_cree2:
.pas_dans_zone:
.pas_zone_entree:
.none_here2:
	lea GWVA_ROOT_OBJ_MOUSEZONE_STRUCT_LENGTH(a1),a1
	dbf d6,.parse_all_enter_mouse_zone

	rts

;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_TEST_XY_IN_XYWH
; Description : Dtermine si une position X1,Y1 se situe dans une zone X2,Y2,W,H
;
; in  : a0.l = pointeur sur un buffer contenant les x2,y2,w,h de la zone
; in  : d0.w = x1
; in  : d1.w = y1
; out : d7.w = GWVA_NO_ERROR_GENERIC si dedans ou
;	   GWVA_ERROR_GENERIC sinon
;
; 28/03/98 : Cration
;------------------------------------------------------------------------------
GWVA_ROOT_TEST_XY_IN_XYWH:
	cmp.w (a0),d0
	blt.s .pas_dans_zone
	cmp.w 2(a0),d1
	blt.s .pas_dans_zone
	move.w (a0),d2
	add.w 4(a0),d2
	cmp.w d2,d0
	bge.s .pas_dans_zone
	move.w 2(a0),d3
	add.w 6(a0),d3
	cmp.w d3,d1
	bge.s .pas_dans_zone
	moveq #GWVA_NO_ERROR_GENERIC,d7
	rts
.pas_dans_zone:
	moveq #GWVA_ERROR_GENERIC,d7
	rts

;------------------------------------------------------------------------------
; Method name : -
; Asm label   : GWVA_ROOT_INTERSEC_ZONES
; Description : Fait l'intersection entre un zone 1 et une zone 2. Le rsultat
; Description : est mis dans la zone 2
;
; in  : a0.l = pointeur sur un buffer contenant les x1,y1,w1,h1 de la zone 1
; in  : a1.l = pointeur sur un buffer contenant les x2,y2,w2,h2 de la zone 2
;
; 28/03/98 : Cration
;------------------------------------------------------------------------------
GWVA_ROOT_INTERSEC_ZONES:
	movem.w (a0),d0-d3
	movem.w (a1),d4-d7
	add.w d0,d2		; x1
	add.w d1,d3		; y1
	add.w d4,d6		; x2
	add.w d5,d7		; y2

; pas de sub.w #1... car on repasse en fin de fonction en w,h

	cmp.w d7,d3
	ble.s .min1
	move.w d7,d3
.min1:	cmp.w d6,d2
	ble.s .min2
	move.w d6,d2
.min2:
	cmp.w d5,d1
	bge.s .max1
	move.w d5,d1
.max1:	cmp.w d4,d0
	bge.s .max2
	move.w d4,d0
.max2:
	sub.w d1,d3
	sub.w d0,d2

	movem.w d0-d3,(a1)
	rts

	endc	; ifge GWVA_ROOT_CTE_MAX_MOUSEZONE-1

	endc	; ifd MDL_EVENTS

;------------------------------------------------------------------------------
